@@ -1,24 +1,24 @@
 package File::Remove;
 
-use 5.005;
+use 5.00503;
 use strict;
-use vars qw(@EXPORT_OK @ISA $VERSION $debug $unlink $rmdir);
-BEGIN {
-	$VERSION   = '1.39_01';
-	@ISA       = qw(Exporter);
-	@EXPORT_OK = qw(remove rm trash); # nothing by default :)
 
-	# Booleanise the debug flag
-	$debug = !! $debug;
+use vars qw{ $VERSION @ISA @EXPORT_OK };
+use vars qw{ $DEBUG $unlink $rmdir    };
+BEGIN {
+	$VERSION   = '1.45_01';
+	$VERSION   = eval $VERSION;
+	@ISA       = qw{ Exporter };
+	@EXPORT_OK = qw{ remove rm clear trash };
 }
 
-# If we ever need a Mac::Glue object,
-# we will want to cache it.
+# If we ever need a Mac::Glue object we will want to cache it.
 my $glue;
 
-use File::Spec ();
 use File::Path ();
 use File::Glob ();
+use File::Spec 3.2701 ();
+use Cwd        3.2701 ();
 
 sub expand (@) {
 	map { -e $_ ? $_ : File::Glob::bsd_glob($_) } @_;
@@ -26,15 +26,19 @@ sub expand (@) {
 
 # $debug variable must be set before loading File::Remove.
 # Convert to a constant to allow debugging code to be pruned out.
-use constant DEBUG => $debug;
+use constant DEBUG    => !! $DEBUG;
 
 # Are we on VMS?
 # If so copy File::Path and assume VMS::Filespec is loaded
-use constant IS_VMS   => $^O eq 'VMS';
+use constant IS_VMS   => !! ( $^O eq 'VMS' );
+
+# Are we on Mac?
+# If so we'll need to do some special trash work
+use constant IS_MAC   => !! ( $^O eq 'darwin' );
 
 # Are we on Win32?
 # If so write permissions does not imply deletion permissions
-use constant IS_WIN32 => $^O eq 'MSWin32';
+use constant IS_WIN32 => !! ( $^O =~ /^MSWin/ or $^O eq 'cygwin' );
 
 
 
@@ -43,25 +47,47 @@ use constant IS_WIN32 => $^O eq 'MSWin32';
 #####################################################################
 # Main Functions
 
+my @END_DELETE = ();
+
+sub clear (@) {
+	my @files = expand( @_ );
+
+	# Do the initial deletion
+	foreach my $file ( @files ) {
+		next unless -e $file;
+		remove( \1, $file );
+	}
+
+	# Delete again at END-time
+	push @END_DELETE, @files;
+}
+
+END {
+	foreach my $file ( @END_DELETE ) {
+		next unless -e $file;
+		remove( \1, $file );
+	}
+}
+
 # acts like unlink would until given a directory as an argument, then
 # it acts like rm -rf ;) unless the recursive arg is zero which it is by
 # default
 sub remove (@) {
 	my $recursive = (ref $_[0] eq 'SCALAR') ? shift : \0;
-	my @files     = expand @_;
+	my @files     = expand(@_);
 
 	# Iterate over the files
 	my @removes;
 	foreach my $path ( @files ) {
-                # need to check for symlink first
-                # could be pointing to nonexisting/non-readable destination
+		# need to check for symlink first
+		# could be pointing to nonexisting/non-readable destination
 		if ( -l $path ) {
-			print "link: $path\n" if $debug;
+			print "link: $path\n" if DEBUG;
 			if ( $unlink ? $unlink->($path) : unlink($path) ) {
 				push @removes, $path;
 			}
 			next;
-                }
+		}
 		unless ( -e $path ) {
 			print "missing: $path\n" if DEBUG;
 			push @removes, $path; # Say we deleted it
@@ -108,7 +134,15 @@ sub remove (@) {
 
 		} elsif ( -d $path ) {
 			print "dir: $path\n" if DEBUG;
-			my $dir = File::Spec->canonpath( $path );
+			my $dir = File::Spec->canonpath($path);
+
+			# Do we need to move our cwd out of the location
+			# we are planning to delete?
+			my $chdir = _moveto($dir);
+			if ( length $chdir ) {
+				chdir($chdir) or next;
+			}
+
 			if ( $$recursive ) {
 				if ( File::Path::rmtree( [ $dir ], DEBUG, 0 ) ) {
 					# Failed to delete the directory
@@ -147,20 +181,26 @@ sub trash (@) {
 		$unlink = $options{unlink};
 		$rmdir  = $options{rmdir};
 
-	} elsif ( $^O eq 'cygwin' || $^O =~ /^MSWin/ ) {
+	} elsif ( IS_WIN32 ) {
+		local $@;
 		eval 'use Win32::FileOp ();';
 		die "Can't load Win32::FileOp to support the Recycle Bin: \$@ = $@" if length $@;
 		$unlink = \&Win32::FileOp::Recycle;
 		$rmdir  = \&Win32::FileOp::Recycle;
 
-	} elsif ( $^O eq 'darwin' ) {
+	} elsif ( IS_MAC ) {
 		unless ( $glue ) {
+			local $@;
 			eval 'use Mac::Glue ();';
 			die "Can't load Mac::Glue::Finder to support the Trash Can: \$@ = $@" if length $@;
 			$glue = Mac::Glue->new('Finder');
 		}
 		my $code = sub {
-			my @files = map { Mac::Glue::param_type(Mac::Glue::typeAlias() => $_) } @_;
+			my @files = map {
+				Mac::Glue::param_type(
+					Mac::Glue::typeAlias() => $_
+				)
+			} @_;
 			$glue->delete(\@files);
 		};
 		$unlink = $code;
@@ -168,13 +208,54 @@ sub trash (@) {
 	} else {
 		die "Support for trash() on platform '$^O' not available at this time.\n";
 	}
-	goto &remove;
+
+	remove(@_);
 }
 
 sub undelete (@) {
 	goto &trash;
 }
 
+
+
+
+
+######################################################################
+# Support Functions
+
+# Do we need to move to a different directory to delete a directory,
+# and if so which.
+sub _moveto {
+	# Do everything in absolute terms
+	my $cwd    = Cwd::abs_path( Cwd::cwd() );
+	my $remove = Cwd::abs_path( File::Spec->rel2abs(shift) );
+
+	# If we are on a different volume we don't need to move
+	my ( $cv, $cd ) = File::Spec->splitpath( $cwd,    1 );
+	my ( $rv, $rd ) = File::Spec->splitpath( $remove, 1 );
+	return '' unless $cv eq $rv;
+
+	# If we have to move, it's to one level above the deletion
+	my @cd = File::Spec->splitdir($cd);
+	my @rd = File::Spec->splitdir($rd);
+	pop @rd;
+
+	# Is the current directory inside of the moveto directory?
+	unless ( @cd > @rd ) {
+		return '';
+	}
+	foreach ( 0 .. $#rd ) {
+		$cd[$_] eq $rd[$_] or return '';
+	}
+
+	# Confirmed, the current working dir is in the removal dir
+	return File::Spec->catpath(
+		$rv,
+		File::Spec->catdir(@rd),
+		''
+	);
+}
+
 1;
 
 __END__
@@ -187,19 +268,19 @@ File::Remove - Remove files and directories
 
 =head1 SYNOPSIS
 
-    use File::Remove qw(remove);
+    use File::Remove 'remove';
 
     # removes (without recursion) several files
-    remove qw( *.c *.pl );
+    remove( '*.c', '*.pl' );
 
     # removes (with recursion) several directories
-    remove \1, qw( directory1 directory2 ); 
+    remove( \1, qw{directory1 directory2} ); 
 
     # removes (with recursion) several files and directories
-    remove \1, qw( file1 file2 directory1 *~ );
+    remove( \1, qw{file1 file2 directory1 *~} );
 
     # trashes (with support for undeleting later) several files
-    trash qw( *~ );
+    trash( '*~' );
 
 =head1 DESCRIPTION
 
@@ -214,9 +295,7 @@ hashref.
 
 =head1 SUBROUTINES
 
-=over 4
-
-=item remove
+=head2 remove
 
 Removes files and directories.  Directories are removed recursively like
 in B<rm -rf> if the first argument is a reference to a scalar that
@@ -228,12 +307,20 @@ In list context it returns a list of files/directories removed, in
 scalar context it returns the number of files/directories removed.  The
 list/number should match what was passed in if everything went well.
 
-=item rm
+=head2 rm
 
 Just calls B<remove>.  It's there for people who get tired of typing
 B<remove>.
 
-=item trash
+=head2 clear
+
+The C<clear> function is a version of C<remove> designed for
+use in test scripts. It takes a list of paths that it will both
+initially delete during the current test run, and then further
+flag for deletion at END-time as a convenience for the next test
+run.
+
+=head2 trash
 
 Removes files and directories, with support for undeleting later.
 Accepts an optional "other platforms" hashref, passing the remaining
@@ -260,22 +347,31 @@ will be called with the filenames that are to be deleted.
 
 =back
 
-=back
+=head1 SUPPORT
 
-=head1 BUGS
+Bugs should always be submitted via the CPAN bug tracker
 
-See http://rt.cpan.org/NoAuth/Bugs.html?Dist=File-Remove for the
-up-to-date bug listing.
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=File-Remove>
+
+For other issues, contact the maintainer.
 
 =head1 AUTHOR
 
-Taken over by Adam Kennedy E<lt>adamk@cpan.orgE<gt>, to fix the
-"deep readonly files" bug, and do some more cleaning up.
+Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
-Taken over by Richard Soderberg E<lt>perl@crystalflame.netE<gt>, so as
-to port it to L<File::Spec> and add tests.
+=head1 COPYRIGHT
 
-Original copyright: (c) 1998 by Gabor Egressy, E<lt>gabor@vmunix.comE<gt>.
+Some parts copyright 2006 - 2011 Adam Kennedy.
+
+Taken over by Adam Kennedy E<lt>adamk@cpan.orgE<gt> to fix the
+"deep readonly files" bug, and do some package cleaning.
+
+Some parts copyright 2004 - 2005 Richard Soderberg.
+
+Taken over by Richard Soderberg E<lt>perl@crystalflame.netE<gt> to
+port it to L<File::Spec> and add tests.
+
+Original copyright: 1998 by Gabor Egressy, E<lt>gabor@vmunix.comE<gt>.
 
 This program is free software; you can redistribute and/or modify it under
 the same terms as Perl itself.
